@@ -6,9 +6,10 @@
 package Lemonldap::NG::Portal::_CAS;
 
 use strict;
-use LWP::UserAgent;
+use Lemonldap::NG::Portal::_Browser;
 
-our $VERSION = '1.0.0';
+our @ISA     = (qw(Lemonldap::NG::Portal::_Browser));
+our $VERSION = '1.3.0';
 
 ## @method hashref getCasSession(string id)
 # Try to recover the CAS session corresponding to id and return session datas
@@ -25,7 +26,7 @@ sub getCasSession {
 
         # Session not available
         if ($id) {
-            $self->lmLog( "CAS session $id isn't yet available", 'info' );
+            $self->_sub( 'userInfo', "CAS session $id isn't yet available" );
         }
         else {
             $self->lmLog( "Unable to create new CAS session: $@", 'error' );
@@ -238,11 +239,6 @@ sub deleteCasSession {
 sub callPgtUrl {
     my ( $self, $pgtUrl, $pgtIou, $pgtId ) = splice @_;
 
-    # LWP User Agent
-    my $ua = new LWP::UserAgent;
-    push @{ $ua->requests_redirectable }, 'POST';
-    $ua->env_proxy();
-
     # Build URL
     my $url = $pgtUrl;
     $url .= ( $pgtUrl =~ /\?/ ? '&' : '?' );
@@ -251,11 +247,10 @@ sub callPgtUrl {
     $self->lmLog( "Call URL $url", 'debug' );
 
     # GET URL
-    my $response = $ua->get($url);
+    my $response = $self->ua()->get($url);
 
     # Return result
     return $response->is_success();
-
 }
 
 1;
