@@ -7,10 +7,10 @@ package Lemonldap::NG::Portal::_SAML;
 
 use strict;
 use Lemonldap::NG::Common::Conf::SAML::Metadata;
+use Lemonldap::NG::Portal::_Browser;
 use XML::Simple;
 use MIME::Base64;
 use String::Random;
-use LWP::UserAgent;        # SOAP call
 use HTTP::Request;         # SOAP call
 use POSIX qw(strftime);    # Convert SAML2 date into timestamp
 use Time::Local;           # Convert SAML2 date into timestamp
@@ -20,7 +20,8 @@ use URI;                   # Get metadata URL path
 # Special comments for doxygen
 #inherits Lemonldap::NG::Common::Conf::SAML::Metadata protected service_metadata
 
-our $VERSION = '1.2.2_01';
+our @ISA     = (qw(Lemonldap::NG::Portal::_Browser));
+our $VERSION = '1.3.0';
 our $samlCache;
 our $initGlibDone;
 
@@ -45,7 +46,7 @@ BEGIN {
     # Load Lasso.pm
     eval 'use Lasso;';
     if ($@) {
-        print STDERR "Lasso.pm not loaded :$@";
+        print STDERR "Lasso.pm not loaded: $@";
         eval 'use constant LASSO => 0;use constant BADLASSO => 0;';
     }
     else {
@@ -1177,8 +1178,13 @@ sub storeRelayState {
     # Session type
     $samlSessionInfo->{_type} = "relaystate";
 
-    # UNIX time
-    $samlSessionInfo->{_utime} = time();
+    # Set _utime for session autoremove
+    # Use default session timeout and relayState session timeout to compute it
+    my $time                  = time();
+    my $timeout               = $self->{timeout};
+    my $samlRelayStateTimeout = $self->{samlRelayStateTimeout} || $timeout;
+
+    $samlSessionInfo->{_utime} = $time + ( $samlRelayStateTimeout - $timeout );
 
     # Store infos in relaystate session
     foreach ( keys %$infos ) {
@@ -1662,10 +1668,6 @@ sub resolveArtifact {
     my ( $self, $profile, $artifact, $method ) = splice @_;
     my $message;
 
-    # LWP User Agent
-    my $ua = new LWP::UserAgent;
-    push @{ $ua->requests_redirectable }, 'POST';
-
     # Login profile
     if ( $profile->isa("Lasso::Login") ) {
 
@@ -1691,7 +1693,7 @@ sub resolveArtifact {
             'debug' );
 
         # SOAP call
-        my $soap_answer = $ua->request($request);
+        my $soap_answer = $self->ua()->request($request);
         if ( $soap_answer->code() == "200" ) {
             $message = $soap_answer->content();
             $self->lmLog( "Get message $message", 'debug' );
@@ -1898,10 +1900,6 @@ sub sendSOAPMessage {
     my ( $self, $endpoint, $message ) = splice @_;
     my $response;
 
-    # LWP User Agent
-    my $ua = new LWP::UserAgent;
-    push @{ $ua->requests_redirectable }, 'POST';
-
     my $request = HTTP::Request->new( 'POST' => $endpoint );
     $request->content_type('text/xml');
     $request->content($message);
@@ -1909,7 +1907,7 @@ sub sendSOAPMessage {
     $self->lmLog( "Send SOAP message $message to $endpoint", 'debug' );
 
     # SOAP call
-    my $soap_answer = $ua->request($request);
+    my $soap_answer = $self->ua()->request($request);
     if ( $soap_answer->code() == "200" ) {
         $response = $soap_answer->content();
         $self->lmLog( "Get response $response", 'debug' );
@@ -3337,15 +3335,57 @@ L<Lemonldap::NG::Portal::AuthSAML>, L<Lemonldap::NG::Portal::UserDBSAML>
 
 =head1 AUTHOR
 
-Xavier Guimard, E<lt>x.guimard@free.frE<gt>,
-Clement Oudot, E<lt>coudot@linagora.comE<gt>
+=over
+
+=item Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
+
+=item François-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
+
+=item Xavier Guimard, E<lt>x.guimard@free.frE<gt>
+
+=item Sandro Cazzaniga, E<lt>cazzaniga.sandro@gmail.comE<gt>
+
+=item Thomas Chemineau, E<lt>thomas.chemineau@gmail.comE<gt>
+
+=back
+
+=head1 BUG REPORT
+
+Use OW2 system to report bug or ask for features:
+L<http://jira.ow2.org>
+
+=head1 DOWNLOAD
+
+Lemonldap::NG is available at
+L<http://forge.objectweb.org/project/showfiles.php?group_id=274>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2009, 2010 by Xavier Guimard, Clement Oudot
+=over
+
+=item Copyright (C) 2009, 2010, 2011, 2012 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
+
+=item Copyright (C) 2012 by Sandro Cazzaniga, E<lt>cazzaniga.sandro@gmail.comE<gt>
+
+=item Copyright (C) 2012 by François-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
+
+=item Copyright (C) 2010, 2011, 2012, 2013 by Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
+
+=item Copyright (C) 2010, 2011 by Thomas Chemineau, E<lt>thomas.chemineau@gmail.comE<gt>
+
+=back
 
 This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.10.0 or,
-at your option, any later version of Perl 5 you may have available.
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2, or (at your option)
+any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see L<http://www.gnu.org/licenses/>.
 
 =cut
