@@ -8,7 +8,7 @@ package Lemonldap::NG::Portal::_WebForm;
 use Lemonldap::NG::Portal::Simple qw(:all);
 use strict;
 
-our $VERSION = '1.2.2_01';
+our $VERSION = '1.3.0';
 
 ## @apmethod int authInit()
 # Does nothing.
@@ -47,33 +47,45 @@ sub extractFormInfo {
             && ( $self->{confirmpassword} = $self->param('confirmpassword') ) );
     }
 
-    # 4. If the captcha feature is enabled, captcha form
-    if ( $self->{captcha_enabled} ) {
-        my $captcha_user_code;
-        if ( $self->param('captcha_user_code') && $self->param('captcha_code') )
-        {
-            $captcha_user_code = $self->param('captcha_user_code');
-            $self->{captcha_code} = $self->param('captcha_code');
+    # 4. Captcha for login form
+    if ( $self->{captcha_login_enabled} && $defUser && $defPassword ) {
+        $self->{captcha_user_code}  = $self->param('captcha_user_code');
+        $self->{captcha_check_code} = $self->param('captcha_code');
+
+        unless ( $self->{captcha_user_code} && $self->{captcha_check_code} ) {
+            $self->lmLog( "Captcha not filled", 'warn' );
+            return PE_CAPTCHAEMPTY;
         }
-        $self->{captcha_result} =
-          $self->checkCaptcha( $captcha_user_code, $self->{captcha_code} );
-        if ( $self->{captcha_result} != 1 ) {
-            if (   $self->{captcha_result} == -3
-                or $self->{captcha_result} == -2 )
+
+        $self->lmLog(
+            "Captcha data received: "
+              . $self->{captcha_user_code} . " and "
+              . $self->{captcha_check_code},
+            'debug'
+        );
+
+        # Check captcha
+        my $captcha_result = $self->checkCaptcha( $self->{captcha_user_code},
+            $self->{captcha_check_code} );
+
+        if ( $captcha_result != 1 ) {
+            if (   $captcha_result == -3
+                or $captcha_result == -2 )
             {
-                $self->lmLog( "Captcha failed: wrong code", 'error' );
+                $self->lmLog( "Captcha failed: wrong code", 'warn' );
                 return PE_CAPTCHAERROR;
             }
-            elsif ( $self->{captcha_result} == 0 ) {
+            elsif ( $captcha_result == 0 ) {
                 $self->lmLog( "Captcha failed: code not checked (file error)",
-                    'error' );
+                    'warn' );
                 return PE_CAPTCHAERROR;
             }
-            elsif ( $self->{captcha_result} == -1 ) {
-                $self->lmLog( "Captcha failed: code has expired", 'error' );
+            elsif ( $captcha_result == -1 ) {
+                $self->lmLog( "Captcha failed: code has expired", 'warn' );
                 return PE_CAPTCHAERROR;
             }
         }
+        $self->lmLog( "Captcha code verified", 'debug' );
     }
 
     # Other parameters
@@ -119,17 +131,22 @@ sub setAuthSessionInfo {
 
 ## @method int checkCaptcha(code, ccode)
 # Check captcha auth
-# @return a constant
 # @param code that user enter in the form
 # @param captcha code generated by Authen::Captcha
+# @return a constant
 sub checkCaptcha {
     my ( $self, $code, $ccode ) = splice @_;
+
     opendir( OUTPUT, $self->{captcha_output} )
       or $self->lmLog( "Can't open captcha output dir", "error" );
     opendir( DATA, $self->{captcha_data} )
       or $self->lmLog( "Can't open captcha data dir", "error" );
-    $self->{captcha_result} = $self->{captcha}->check_code( $code, $ccode );
+
+    my $result = $self->{captcha}->check_code( $code, $ccode );
+
     closedir(OUTPUT) && closedir(DATA);
+
+    return $result;
 }
 
 1;
